#include "camera.h"

Camera::Camera(QVector3D position, QVector3D up, float yaw, float pitch) :
    Front(QVector3D(0.0f, 0.0f, -1.0f)),
    MovementSpeed(CAM_SPEED),
    MouseSensitivity(CAM_SENSITIVITY),
    Zoom(CAM_ZOOM),
    Position(position),
    WorldUp(up),
    Yaw(yaw),
    Pitch(pitch)
{
    updateCameraVectors();
}

Camera::Camera(float posX, float posY, float posZ,
               float upX, float upY, float upZ, float yaw, float pitch) :
    Front(QVector3D(0.0f, 0.0f, -1.0f)),
    MovementSpeed(CAM_SPEED),
    MouseSensitivity(CAM_SENSITIVITY),
    Zoom(CAM_ZOOM),
    Position(QVector3D(posX, posY, posZ)),
    WorldUp(QVector3D(upX, upY, upZ)),
    Yaw(yaw),
    Pitch(pitch)
{
    updateCameraVectors();
}

QMatrix4x4 Camera::GetViewMatrix()
{
    QMatrix4x4 view;
    view.lookAt(Position, Position + Front, Up);
    return view;
}

void Camera::ProcessKeyboard(Camera_Movement direction, float deltaTime)
{
    float velocity = MovementSpeed * deltaTime;
    if (direction == FORWARD)
        Position += Front * velocity;
    if (direction == BACKWARD)
        Position -= Front * velocity;
    if (direction == LEFT)
        Position -= Right * velocity;
    if (direction == RIGHT)
        Position += Right * velocity;
}

void Camera::ProcessMouseMovement(float xoffset, float yoffset, bool constrainPitch)
{
    xoffset *= MouseSensitivity;
    yoffset *= MouseSensitivity;

    Yaw   += xoffset;
    Pitch += yoffset;

    // make sure that when pitch is out of bounds, screen doesn't get flipped
    if (constrainPitch)
    {
        if (Pitch > 89.0f)
            Pitch = 89.0f;
        if (Pitch < -89.0f)
            Pitch = -89.0f;
    }

    // update Front, Right and Up Vectors using the updated Euler angles
    updateCameraVectors();
}

void Camera::ProcessMouseScroll(float yoffset)
{
    Zoom -= yoffset;
    if (Zoom < 1.0f)
        Zoom = 1.0f;
    // if (Zoom > 45.0f)
    //     Zoom = 45.0f;
}

void Camera::updateCameraVectors()
{
    // calculate the new Front vector
    QVector3D front;
    front.setX(cos(qDegreesToRadians(Yaw)) * cos(qDegreesToRadians(Pitch)));
    front.setY(sin(qDegreesToRadians(Pitch)));
    front.setZ(sin(qDegreesToRadians(Yaw)) * cos(qDegreesToRadians(Pitch)));
    Front = front.normalized();

    // also re-calculate the Right and Up vector
    // normalize the vectors, because their length gets closer to 0
    // the more you look up or down which results in slower movement.
    Right = QVector3D::crossProduct(Front, WorldUp).normalized();
    Up = QVector3D::crossProduct(Right, Front).normalized();
}
