#include "polyhedradialog.h"
#include "ui_polyhedradialog.h"
#if defined(JAV_WIDGET)
#include "crystalviewwidget.h"
#else
#include "javwindow.h"
#endif

#include <QStringListModel>
#include <QDebug>
#include <QString>
#include <QStringList>
#include <QInputDialog>

extern "C" float getbonddistance_(int *val1, int *val2);
extern "C" void ges_add_fragments_win_(int *code, int *wm, int *kgeom=nullptr,
       int *kspec1=nullptr, int *kspec2=nullptr, int *kpos=nullptr, float *bdist=nullptr, int *kadd=nullptr, int *nfrag=0);

static const QStringList polyhedraList = {"Tetrahedron", "Octahedron", "Square Plane",
                             "Cube", "Trigonal Plane", "Antiprism Tetragonal",
                             "Prism Trigonal", "Icosahedron", "Isolated Atoms"};

PolyhedraDialog::PolyhedraDialog(QWidget *parent) :
    QDialog(parent),
    add(false),
    nspec(0),
    lastOrigin(0),
    lastVertex(0),
    ui(new Ui::PolyhedraDialog)
  {
#if defined(JAV_WIDGET)
    jav = qobject_cast<CrystalViewWidget*>(parent);
#else
    jav = qobject_cast<JavWindow*>(parent);
#endif
    ui->setupUi(this);
    mTableWindow = new TableWindow(this);

    //Fill combo with polyhedra list
    QStringListModel *model = new QStringListModel();
    model->setStringList(polyhedraList);
    ui->PolyhedraCombo->setModel(model);

    connect(ui->buttonBox, &QDialogButtonBox::accepted, this, &PolyhedraDialog::buttonBoxAccepted);
    connect(ui->buttonBox, &QDialogButtonBox::rejected, this, &PolyhedraDialog::buttonBoxRejected);
    connect(ui->radioRandom, &QRadioButton::clicked, this, &PolyhedraDialog::radioRandomClicked);
    connect(ui->radioCentered, &QRadioButton::clicked, this, &PolyhedraDialog::radioCenteredClicked);
    connect(ui->PolyhedraCombo, &QComboBox::activated, this, &PolyhedraDialog::polyhedraComboActivated);
    connect(ui->originCombo, &QComboBox::activated, this, &PolyhedraDialog::originComboActivated);
    connect(ui->verticesCombo, &QComboBox::activated, this, &PolyhedraDialog::verticesComboActivated);
}

PolyhedraDialog::~PolyhedraDialog()
{
    delete ui;
}

void PolyhedraDialog::set_visible_widgets(int geom, bool add)
{
    QString polyhedron = polyhedraList.at(geom);
    setWindowTitle("Add "+polyhedron);

    ui->distanceLabel->setVisible(geom != ISOLATED);
    ui->distanceSpinBox->setVisible(geom != ISOLATED);
    ui->verticesLabel->setVisible(geom != ISOLATED);
    ui->verticesCombo->setVisible(geom != ISOLATED);

    if (geom == ISOLATED) {
        ui->positionBox->setVisible(false);
        ui->originLabel->setText("Element:");
    } else {
        set_distance();
        ui->positionBox->setVisible(add);
        ui->originLabel->setText("Origin:");
    }

    //Resize window after removing/adding frame 'Position'
    QSize size = sizeHint();
    resize(size);
}

void PolyhedraDialog::set_polyhedron_type(int geom, bool add)
{
    QString polyhedron = polyhedraList.at(geom);
    ui->PolyhedraCombo->setCurrentText(polyhedron);
    this->add = add;

    int icod = 0;
    int whichModel=1;
    int idum, kspec1, kspec2;
    ges_add_fragments_win_(&icod, &whichModel, &idum, &kspec1, &kspec2);

    //Create models for combo
    modelspec = new QStringListModel();
    modelatom = new QStringListModel();

    //Fill combo with list of species if empty
    QStringList specList = jav->FillSpecies(add);
    specList << "Other...";
    modelspec->setStringList(specList);
    ui->originCombo->setModel(modelspec);
    ui->verticesCombo->setModel(modelspec);

    //Set starting position for combo
    ui->originCombo->setCurrentIndex(kspec1-1);
    ui->verticesCombo->setCurrentIndex(kspec2-1);

    //Save combo selection
    nspec = modelspec->rowCount()-1;
    lastOrigin = ui->originCombo->currentIndex();
    lastVertex = ui->verticesCombo->currentIndex();

    //Set frame 'Position'
    if (add) {
        QStringList atomList = jav->FillAtoms();
        modelatom->setStringList(atomList);
        ui->atomsCombo->setModel(modelatom);
        ui->atomsCombo->setEnabled(ui->radioCentered->isChecked());
    }

    set_visible_widgets(geom, add);
}

bool PolyhedraDialog::id_specie_from_combo(const QComboBox *combo, int &id) {
    //Convert selected specie in a valid id for the list of species (-Z if absent in the list)
    int index = combo->currentIndex();
    id = 0;
    if (index == -1 || index == modelspec->rowCount()-1) return false; //Empty selection or selection on 'Other..'
    id = index+1;
    if (id > nspec) id = -is_valid_symbol(combo->currentText().toStdString());
    return true;
}

void PolyhedraDialog::buttonBoxAccepted()
{
    int icod = 3;
    int whichModel=1;
    int kgeom = ui->PolyhedraCombo->currentIndex()+1;
    int kspec1;
    if (!id_specie_from_combo(ui->originCombo, kspec1) )return;
    int kadd = add ? 1 : 0;
    int natoms = ui->numberSpinBox->value();
    if (ui->PolyhedraCombo->currentIndex() == ISOLATED) {        
        int dum = 0;
        float fdum = 0.f;
        ges_add_fragments_win_(&icod, &whichModel, &kgeom, &kspec1, &dum,
                                  &dum, &fdum, &kadd, &natoms);

    } else {
        int kspec2;
        if (!id_specie_from_combo(ui->verticesCombo, kspec2) )return;
        float bdist = static_cast<float>(ui->distanceSpinBox->value());
        int kpos = 0;
        if (kadd) {
            if (ui->radioCentered->isChecked())
                kpos = ui->atomsCombo->currentIndex()+1;
        }
        ges_add_fragments_win_(&icod, &whichModel, &kgeom, &kspec1, &kspec2,
                                  &kpos, &bdist, &kadd, &natoms);
    }
    icod = 1; //Quit dialog saving action
    ges_add_fragments_win_(&icod, &whichModel);
}

int PolyhedraDialog::add_element_to_modelspec(QString newel)
{
    //check if newel is already present in the model
    QModelIndexList indexx = modelspec->match(modelspec->index(0,0),Qt::DisplayRole,newel,1,Qt::MatchExactly);
    if (!indexx.count()) {        
        //add new element to the model, combo will be automatically updated
        int row = modelspec->rowCount();
        modelspec->insertRows(row-1,1);
        QModelIndex index = modelspec->index(row-1);
        modelspec->setData(index,newel);

        return index.row();        
    } else {                
        return indexx.at(0).row();                
    }
}

void PolyhedraDialog::set_distance()
{
    //Check for valid selections
    int nrows = modelspec->rowCount();
    if (nrows == 1) return; //The model contains only 'Other..'
    int indexO;
    if (!id_specie_from_combo(ui->originCombo, indexO) )return;
    int indexV;
    if (!id_specie_from_combo(ui->verticesCombo, indexV) )return;

    //Update label with chemical species
    QString eleO = ui->originCombo->currentText();
    QString eleV = ui->verticesCombo->currentText();
    QString dist = QString("%1 - %2 distance:").arg(eleO).arg(eleV);
    ui->distanceLabel->setText(dist);

    //Update suggested bond distance
    double distance = static_cast<double>(getbonddistance_(&indexO,&indexV)); //FIX +1 when is_valid_symbol is called
    ui->distanceSpinBox->setValue(distance);
}

void PolyhedraDialog::radioRandomClicked()
{
    ui->atomsCombo->setEnabled(false);
}

void PolyhedraDialog::radioCenteredClicked()
{
    ui->atomsCombo->setEnabled(true);
}

void PolyhedraDialog::polyhedraComboActivated(int index)
{
    set_visible_widgets(index,this->add);
}

//void PolyhedraDialog::on_buttonBox_clicked(QAbstractButton *button)
//{
//    if ((QPushButton *)button == ui->buttonBox->button(QDialogButtonBox::Ok)){
//        qDebug() << "Button OK";
//    } else {
//        qDebug() << "Button Cancel";
//    }
//}

void PolyhedraDialog::buttonBoxRejected()
{
    int icod = 1;
    int whichModel=1;
    ges_add_fragments_win_(&icod, &whichModel);
}

void PolyhedraDialog::originComboActivated(int index)
{
    if (index == lastOrigin) return;
    if (ui->originCombo->currentText() == "Other...") {
        bool ok;
        QString newel = mTableWindow->GetElement(&ok);
        //QString newel = QInputDialog::getText(this, "Other...",
        //                            "Enter chemical element:", QLineEdit::Normal,
        //                            "",&ok);
        if (ok && !newel.isEmpty()) {
            if (!is_valid_symbol(newel.toStdString())) return;
            int index = add_element_to_modelspec(newel);            
            ui->originCombo->setCurrentIndex(index);
        } else {
            ui->originCombo->setCurrentIndex(lastOrigin);
        }
    }
    lastOrigin = ui->originCombo->currentIndex();
    set_distance();
}

void PolyhedraDialog::verticesComboActivated(int index)
{
    if (index == lastVertex) return;
    if (ui->verticesCombo->currentText() == "Other...") {
        bool ok;
        QString newel = mTableWindow->GetElement(&ok);

//        QString newel = QInputDialog::getText(this, "Other...",
//                                    "Enter chemical element:", QLineEdit::Normal,
//                                    "",&ok);
        if (ok && !newel.isEmpty()) {
            if (!is_valid_symbol(newel.toStdString())) return;
            int index = add_element_to_modelspec(newel);
            ui->verticesCombo->setCurrentIndex(index);
        } else {
            ui->verticesCombo->setCurrentIndex(lastVertex);
        }
    }
    lastVertex = ui->verticesCombo->currentIndex();
    set_distance();
}
